/*--------------------------------------------------------------------.
| execute.c							      |
| 								      |
| Copyright (c) 1997 Jonathan Hudson				      |
| 								      |
| You may distribute under the terms of either the GNU General Public |
| License or the Artistic License, as specified in the README file.   |
`--------------------------------------------------------------------*/

static char rcsid[] = "$Id: execute.c,v 1.6 1997/11/30 21:08:55 jrh Exp $";

/*
 * $Log: execute.c,v $
 * Revision 1.6  1997/11/30 21:08:55  jrh
 * Added HOT_DO functions
 *
 * Revision 1.5  1997/11/15 14:33:02  jrh
 * reindented for release 1.05
 *
 * Revision 1.4  1997/10/12 20:41:40  jrh
 * indented source code
 *
 * Revision 1.3  1997/10/05 15:58:59  jrh
 * Changed some comments
 *
 * Revision 1.2  1997/10/04 19:55:03  jrh
 * tidied up for release v1.0
 *
 * Revision 1.1  1997/10/02 21:28:29  jrh
 * Initial revision
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdarg.h>
#include <string.h>
#include <things.h>
#include <qdos.h>
#include <qptr.h>
#include "qascade.h"

/*------------------------------------------------------------------------.
| Try and execute an executable thing. We can pass arguments, timeouts,	  |
| channels etc. Someone once told me that the documentation on doing such |
| things was excellent and comprehensive, so you'll know where to look.	  |
| 									  |
| If was really that easy, everyone would be doing it ....		  |
`------------------------------------------------------------------------*/

int TryThing (char *thing, char *cmd, short timo, void **pt, int nfn,...)
{
    int sts;
    THING_HEADER_EXEC *t;
    long vers;
    THING_LINKAGE *p;
    struct JOBHEADER *jh;	/* ha ha */

    *pt = NULL;
    sts = (int) sms_uthg (thing, -1, -1, NULL, NULL, (long *) &vers, &p);
    if (sts > 0)
    {
	t = (THING_HEADER_EXEC *) sts;
	if (t->thh.thh_type == THT_EXEC)
	{
	    long sa;
	    long ha;
	    void *jb, *stk;
	    short elen, nlen, k;

	    sa = (long) t + t->thh_strt;
	    ha = (long) t + t->thh_hdrs;

	    sts = mt_cjob (t->thh_hdrl, t->thh_data + 4, (char *) sa, 0, &jb);
	    if (sts > 0)
	    {
		va_list va;
		char **fn;
		va_start (va, nfn);
		fn = va_arg (va, char **);

		memcpy (jb, (void *) ha, t->thh_hdrl);
		jh = (((struct JOBHEADER *) jb) - 1);
		stk = (void *) jh->jb_regs.jb_A7;
		if (cmd && (nlen = strlen (cmd)))
		{
		    elen = (nlen + 1) & ~1;
		}
		else
		{
		    nlen = elen = 0;
		}
		if (nfn)
		{
		    elen = elen + (nfn << 2);	/* channels */
		}

		if (elen)
		{
		    ((char *) stk) -= elen;
		    (void *) jh->jb_regs.jb_A7 = stk;
		}

		*((short *) stk)++ = nfn;

		for (k = 0; k < nfn; k++)
		{
		    int flg;
		    flg = k ? 3 : 0;
		    *((long *) stk)++ = io_open3 (*(fn + k), flg, sts);
		}
		*((short *) stk)++ = nlen;
		while (nlen--)
		{
		    *((char *) stk)++ = *cmd++;
		}
		sts = mt_activ (sts, 8, 0);	/* WAS timo */
		sms_fthg (thing, -1, NULL, 0, NULL, NULL);
	    }
	    *pt = t;
	}
	else
	{
	    sts = -15;
	}
    }
    else
    {
	sts = -7;
    }
    return sts;
}

/*------------------------------------------------------.
| Attempt to determine the PROG and DATA directories:   |
| as a CStart = main program, we must do this ourselves |
`------------------------------------------------------*/

static void *xgetcwd (char *s)
{
    extern char *_sys_var;
    struct QLSTR **q;
    char *d = NULL;

    if (q = (struct QLSTR ** q) (_sys_var + 0xAC))
    {
	d = s;
	for (d = s; s != d + 80; s += 40, q++)
	{
	    strncpy (s, (*q)->qs_str, (*q)->qs_strlen);
	    *(s + (*q)->qs_strlen) = 0;
	}
    }
    return d;
}

/*-------------------------------------------------------.
| start an executable program with arguments		 |
| (10 minutes or the full half hour).			 |
`-------------------------------------------------------*/

int fork (char *prog, char *cmd)
{
    chanid_t chid;
    int jobid, rv = -1;

    if ((rv = io_open (prog, 0)) < 0)
    {
	char *fn, *dd, fnam[40], defdirs[80];
	if (xgetcwd (defdirs))
	{
	    short j;

	    for (dd = defdirs; rv < 0 && dd != defdirs + 80; dd += 40)
	    {
		fn = stpcpy (fnam, dd);
		stpcpy (fn, prog);
		rv = io_open (fnam, 0);
	    }
	}
    }

    if (rv > 0)
    {
	struct qdirect qdir;
	void *jobaddr;
	struct JOBHEADER *jh;	/* ha ha */
	long codsiz;
	short elen, nlen;

	chid = rv;
	if ((rv = fs_headr (chid, -1, &qdir, sizeof (qdir))) > 0)
	{
	    if (qdir.d_type == QF_PROG_TYPE)
	    {
		jobid = mt_cjob (qdir.d_length, qdir.d_datalen, NULL, 0,
				 &jobaddr);
		if (jobid > 0)
		{
		    void *stk;

		    if ((rv = fs_load (chid, jobaddr, qdir.d_length)) > 0)
		    {
			io_close (chid);
			chid = 0;
			jh = (((struct JOBHEADER *) jobaddr) - 1);
			stk = (void *) jh->jb_regs.jb_A7;
			if (cmd && (nlen = strlen (cmd)))
			{
			    elen = (nlen + 1) & ~1;
			}
			else
			{
			    nlen = elen = 0;
			}

			((char *) stk) -= elen;
			(void *) jh->jb_regs.jb_A7 = stk;
			*((short *) stk)++ = 0;
			*((short *) stk)++ = nlen;
			while (nlen--)
			{
			    *((char *) stk)++ = *cmd++;
			}
			rv = mt_activ (jobid, 8, 0);
		    }
		    else
		    {
			mt_frjob (jobid, rv);
		    }
		}
	    }
	    else
	    {
		rv = -15;
	    }
	}
	if (chid > 0)
	{
	    io_close (chid);
	}
    }
    return rv;
}

void BurnMe(char *p)
{
    char kname[2];
    short key,knum;
    HK_ITEM *h;
    int sts;

    kname[0] = *p;
    kname[1] = '\0';
    sts = hk_fitem(kname, &h, &key, &knum);
    if(sts == 0)
	hk_do(h);
}
